# bitchxpack.tcl v1.33 and earlier by DeathHand <dh@rusclub.ml.org>
# ftp://ftp.rusclub.ml.org/eggdrop/scripts/ctcp/

# v1.34-1.40 (6 December 1998) by slennox <slenny@ozemail.com.au>
# Latest versions can be found at ftp.sodre.net/pub/eggdrop/

# BitchX CTCP and AWAY simulator + Flud Net protection.
# Tested on eggdrop 1.0x, 1.1.x and 1.3.x

# This script combines the best features taken from:
# BitchX.tcl v3 by sinkhole
# tcl.ctcpprot v1.7 by CFusion, Ec|ipse, EraseMe
# fingerctcp.tcl by LSC
# fludprot.tcl v1.0 by DeathHand

# Credits: the script would not be possible without stran9er.

# bitchxpack1.40.tar.gz contains three versions of bitchxpack.tcl v1.40:
# bitchxpack1.40.tcl-74p1+ - simulates BitchX-74p1+
# bitchxpack1.40.tcl-74p4+ - simulates BitchX-74p4+
# bitchxpack1.40.tcl-75p1+ - simulates BitchX-75p1+ <- you are here :-)

# v1.3  - tested on eggdrop 1.3.x and updated to BitchX-74p4+
# v1.31 - fixed a bug with CLIENTINFO OPS reply (found by stran9er)
# v1.32 - fixed a bug with USERINFO reply (found by MSDN)
# v1.33 - fixed a bug with ERRMSG\ECHO reply (found by slennox\whaQ)
# v1.34 - updated to simulate BitchX-75p1+
# v1.35 - corrected some CTCPs which shouldn't reply when sent to a
#         channel (found by Alien88 & _Juggie_)
# v1.36 - now answers only one stacked CTCP, added PING and TIME to
#         ctcp bindings
# v1.37 - fixed an erroneous CTCP reply
# v1.38 - fixed the fix to the erroneous CTCP reply, rewrote the
#         away simulator
# v1.39 - fixed some erroneous replies I found after doing quite a bit
#         of testing, getting pretty close to 100% accurate simulation
#         now (if it isn't already) :-)
# v1.40 - added option to switch off flud protection, added setting to
#         switch on protect-telnet, now comes in three versions to
#         simulate 74p1+, 74p4+, or 75p1+.

# Note: v1.34-1.40 have been tested on eggdrop 1.1.5 and 1.3.22 but
# not on any 1.0x version bot. Thanks to trix for testing on 1.1.5.

# Note: the previous versions failed to work on:
# eggdrop v1.3.13 & v1.3.14 (reported by [aLeXuS]).
# This happened because it could not bind all CTCPs with
# "bind ctcp - * bitchxctcp" (bug?).
# Now all CTCPs are bound separately.

# Please set the following variables

# Flud protection, this is on by default - however if you use some other
# script for responding to CTCP floods you may wish to switch this off
set fludenabled 1

# Will not reply to more than $maxctcpflud ctcps in $maxtimeflud seconds
set maxctcpflud 5
set maxtimeflud 30

# This enables protect-telnet (for people too lazy to switch it on in their
# config file), applicable to 1.3.x bots only
set protect-telnet 1

# You don't have to change anything below

bind ctcp - CLIENTINFO bitchxctcp
bind ctcp - USERINFO bitchxctcp
bind ctcp - VERSION bitchxctcp
bind ctcp - FINGER bitchxctcp
bind ctcp - ERRMSG bitchxctcp
bind ctcp - ECHO bitchxctcp
bind ctcp - INVITE bitchxctcp
bind ctcp - WHOAMI bitchxctcp
bind ctcp - OP bitchxctcp
bind ctcp - OPS bitchxctcp
bind ctcp - UNBAN bitchxctcp
bind ctcp - PING bitchxctcp
bind ctcp - TIME bitchxctcp

proc bitchxctcp {nick uhost handle dest key arg} {

  # flud protection
  global fludenabled
  if {$fludenabled} {
    global fludprot maxctcpflud maxtimeflud botnick
    incr fludprot
    utimer $maxtimeflud fludprotdecr
    if {$fludprot>$maxctcpflud} {
      return 1
    }
  }
  
  # answer one stacked ctcp only
  global onestack
  if {$onestack} {return 1}
  set onestack 1
  utimer 2 "set onestack 0"

  putlog "BitchX: $key $arg CTCP:  from $nick \($uhost\)"

  # ctcp clientinfo
  if {"$key"=="CLIENTINFO"} {
    set bxcmd [string toupper $arg]
    if {$bxcmd == "NONE"} {return 0}
    if {$bxcmd == ""} {set bxcmd "NONE"}
    switch $bxcmd {
      NONE { set text "notice $nick :CLIENTINFO SED UTC ACTION DCC CDCC BDCC XDCC VERSION CLIENTINFO USERINFO ERRMSG FINGER TIME PING ECHO INVITE WHOAMI OP OPS UNBAN IDENT XLINK UPTIME  :Use CLIENTINFO <COMMAND> to get more specific information"}
      SED { set text "notice $nick :CLIENTINFO SED contains simple_encrypted_data"}
      UTC  { set text "notice $nick :CLIENTINFO UTC substitutes the local timezone"}
      ACTION { set text "notice $nick :CLIENTINFO ACTION contains action descriptions for atmosphere"}
      DCC { set text "notice $nick :CLIENTINFO DCC requests a direct_client_connection"}
      CDCC { set text "notice $nick :CLIENTINFO CDCC checks cdcc info for you"}
      BDCC { set text "notice $nick :CLIENTINFO BDCC checks cdcc info for you"}
      XDCC { set text "notice $nick :CLIENTINFO XDCC checks cdcc info for you"}
      VERSION { set text "notice $nick :CLIENTINFO VERSION shows client type, version and environment"}
      CLIENTINFO { set text "notice $nick :CLIENTINFO CLIENTINFO gives information about available CTCP commands"}
      USERINFO { set text "notice $nick :CLIENTINFO USERINFO returns user settable information"}
      ERRMSG { set text "notice $nick :CLIENTINFO ERRMSG returns error messages"}
      FINGER { set text "notice $nick :CLIENTINFO FINGER shows real name, login name and idle time of user"}
      TIME { set text "notice $nick :CLIENTINFO TIME tells you the time on the user's host"}
      PING { set text "notice $nick :CLIENTINFO PING returns the arguments it receives"}
      ECHO { set text "notice $nick :CLIENTINFO ECHO returns the arguments it receives"}
      INVITE { set text "notice $nick :CLIENTINFO INVITE invite to channel specified"}
      WHOAMI { set text "notice $nick :CLIENTINFO WHOAMI user list information"}
      OP { set text "notice $nick :CLIENTINFO OP ops the person if on userlist"}
      OPS { set text "notice $nick :CLIENTINFO OPS ops the person if on userlist"}
      UNBAN { set text "notice $nick :CLIENTINFO UNBAN unbans the person from channel"}
      IDENT { set text "notice $nick :CLIENTINFO IDENT change userhost of userlist"}
      XLINK { set text "notice $nick :CLIENTINFO XLINK x-filez rule"}
      UPTIME { set text "notice $nick :CLIENTINFO UPTIME my uptime"}
      default { set text "notice $nick :ERRMSG CLIENTINFO: $arg is not a valid function"}
    }
    putserv "$text"
    return 1
  }

  # ctcp version
  if {"$key"=="VERSION"} {
    global bxversion system
    putserv "notice $nick :VERSION BitchX-$bxversion by panasync - $system : Keep it to yourself!"
    return 1
  }

  # ctcp userinfo
  if {"$key"=="USERINFO"} {
    putserv "notice $nick :USERINFO "
    return 1
  }

  # ctcp finger
  if {"$key"=="FINGER"} {
    global realname whoami machine jointime
    set idletime [expr [unixtime] - $jointime]
    putserv "notice $nick :FINGER $realname ($whoami@$machine) Idle $idletime seconds"
    return 1
  }

  # ctcp echo
  if {"$key"=="ECHO"} {
    if {[lsearch -exact [string tolower [channels]] [string tolower $dest]] != -1} {return 1}
    set arg [string range $arg 0 59]
    putserv "notice $nick :ECHO $arg"
    return 1
  }

  # ctcp errmsg
  if {"$key"=="ERRMSG"} {
    if {[lsearch -exact [string tolower [channels]] [string tolower $dest]] != -1} {return 1}
    set arg [string range $arg 0 59]
    putserv "notice $nick :ERRMSG $arg"
    return 1
  }
  
  # ctcp Invite
  if {"$key"=="INVITE"} {
    if {[llength $arg] == 0} {return 1}
    if {[lsearch -exact [string tolower [channels]] [string tolower $dest]] != -1} {return 1}
    set chan [lindex $arg 0]
    if {[string index $chan 0] == "#"} {
      if {[lsearch -exact [string tolower [channels]] [string tolower $chan]] != -1} {
        putserv "notice $nick :BitchX: Access Denied"
      } else {
        putserv "notice $nick :BitchX: I'm not on that channel"
      }
    }
    return 1
  }

  # ctcp WhoAmI
  if {"$key"=="WHOAMI"} {
    if {[lsearch -exact [string tolower [channels]] [string tolower $dest]] != -1} {return 1}
    putserv "notice $nick :BitchX: Access Denied"
    return 1
  }

  # ctcp op, ctcp ops
  if {"$key"=="OP" || "$key"=="OPS"} {
    if {[llength $arg] == 0} {return 1}
    if {[lsearch -exact [string tolower [channels]] [string tolower $dest]] != -1} {return 1}
    set chan [lindex $arg 0]
    putserv "notice $nick :BitchX: I'm not on $chan, or I'm not opped"
    return 1
  }

  # ctcp UnBan
  if {"$key"=="UNBAN"} {
    if {[llength $arg] == 0} {return 1}
    if {[lsearch -exact [string tolower [channels]] [string tolower $dest]] != -1} {return 1}
    set chan [lindex $arg 0]
    if {[lsearch -exact [string tolower [channels]] [string tolower $chan]] != -1} {
      putserv "notice $nick :BitchX: Access Denied"
    } else {
      putserv "notice $nick :BitchX: I'm not on that channel"
    }
    return 1
  }

  return 0
}

# execute on connect to a server
proc bitchxjoin {} {
  global botnick jointime isaway
  putserv "MODE $botnick +iw-s"
  set jointime [unixtime]
  set isaway 0
}

# improved away simulator
if {![info exists awaytimer]} {
  set awaytimer 1
  set isaway 0
  timer [expr [rand 300] + 10] bitchxaway
}

proc bitchxaway {} {
  global isaway botnick
  if {!$isaway} {
    putserv "AWAY :is away: (Auto-Away after 10 mins) \[BX-MsgLog On\]"
    set isaway 1
    putlog "bitchxpack: $botnick set AWAY"
  } else {
    putserv "AWAY"
    set isaway 0
    putlog "bitchxpack: $botnick set BACK"
  }
  timer [expr [rand 300] + 10] bitchxaway
}

# fludprot utility
proc fludprotdecr {} {
  global fludprot
  incr fludprot -1
}

set bxversion "75p1+"
set fludprot 0
set onestack 0
set system "*IX"
catch { set system  "[exec uname -s -r]" }
global username
set whoami "$username"
catch { set whoami  "[exec whoami]" }
set machine ""
catch { set machine "[exec uname -n]" }

set init-server ${init-server}\n"bitchxjoin"

putlog "Loaded bitchxpack.tcl v1.40 (eggdrop v1.0x/v1.1.x/v1.3.x)"
putlog "Simulating BitchX-$bxversion on $system"
if {$fludenabled} {
  putlog "Allowing $maxctcpflud CTCPs in $maxtimeflud seconds"
}
